//Functions for managing the main application cycle, registering the callbacks
// to create an application and its completion
#include <app.h>

// Functions for displaying messages in the logging window
#include <dlog.h>

// Functions for creating and managing user interface elements. Here are all the widgets,
// such as: windows, buttons, etc.
#include <Elementary.h>

//Functions for using callback for hardware Back button and for
// using extended widgets for a circle screen
#include <efl_extension.h>

typedef struct _UIData {
   Evas_Object *win;
   Evas_Object *conform;
   Evas_Object *layout;
   Evas_Object *button;
   Evas_Object *label;
   Evas_Object *popup;

   int click_count;
} UIData;

static void
_file_abs_resource_path_get(char *res_file_path, char *abs_path, int buf_size)
{
   char *res_dir_path = app_get_resource_path();

   if (res_dir_path)
     {
        snprintf(abs_path, buf_size, "%s%s", res_dir_path, res_file_path);
        free(res_dir_path);
     }
}

static Evas_Object *
_image_create(Evas_Object *parent, char *image_name)
{
   Evas_Object *icon = elm_image_add(parent);

   char image_path[PATH_MAX] = {0,};
   _file_abs_resource_path_get(image_name, image_path, PATH_MAX);
   elm_image_file_set(icon, image_path, NULL);

   evas_object_show(icon);

   return icon;
}

static Evas_Object *
_popup_button_create(Evas_Object *parent, char *style, char *icon_name)
{
   Evas_Object *button = elm_button_add(parent);

   elm_object_style_set(button, style);

   Evas_Object *btn_icon = _image_create(button, icon_name);
   elm_object_content_set(button, btn_icon);

   evas_object_show(button);

   return button;
}

static void
_remove_popup(UIData *ui)
{
   evas_object_del(ui->popup);
   ui->popup = NULL;
}

static void
_cancel_exit_cb(void *data, Evas_Object *obj, void *info)
{
   UIData *ui = data;

   _remove_popup(ui);
}

static void
_apply_exit_cb(void *data, Evas_Object *obj, void *info)
{
   ui_app_exit();
}

static void
_exit_popup_create(UIData *ui)
{
   ui->popup = elm_popup_add(ui->layout);
   elm_object_style_set(ui->popup, "circle");

   Evas_Object *popup_layout = elm_layout_add(ui->popup);
   elm_layout_theme_set(popup_layout, "layout", "popup", "content/circle/buttons2");
   elm_object_content_set(ui->popup, popup_layout);

   elm_object_part_text_set(popup_layout, "elm.text", "Close the application?");
   elm_object_part_text_set(popup_layout, "elm.text.title", "Confirmation");

   Evas_Object *left_button  = _popup_button_create(popup_layout, "popup/circle/left", "cancel.png");
   elm_object_part_content_set(ui->popup, "button1", left_button);
   evas_object_smart_callback_add(left_button, "clicked", _cancel_exit_cb, ui);

   Evas_Object *right_button = _popup_button_create(popup_layout, "popup/circle/right", "apply.png");
   elm_object_part_content_set(ui->popup, "button2", right_button);
   evas_object_smart_callback_add(right_button, "clicked", _apply_exit_cb, ui);

   evas_object_show(ui->popup);
}

static void
_win_back_cb(void *data, Evas_Object *obj, void *event_info)
{
   UIData *ui = data;

   if (!ui->popup)
     {
        _exit_popup_create(ui);
     }
   else
     {
        _remove_popup(ui);
     }
}

static void
press_cb(void *data, Evas_Object *button, void *event_info)
{
   dlog_print(DLOG_DEBUG, "lesson15", "PRESS event!");
}

static void
repeat_cb(void *data, Evas_Object *button, void *event_info)
{
   dlog_print(DLOG_DEBUG, "lesson15", "REPEAT event!");
}

static void
unpress_cb(void *data, Evas_Object *button, void *event_info)
{
   dlog_print(DLOG_DEBUG, "lesson15", "UNPRESS event!");
}

static void
_label_click_count_text_set(UIData *ui)
{
   const int MAX_TEXT_SIZE  = 1024;
   char text[MAX_TEXT_SIZE] = {0,};

   snprintf(text, MAX_TEXT_SIZE, "<font align=center>Number of clicks: %d</font>", ui->click_count);

   elm_object_text_set(ui->label, text);
}

static void
click_cb(void *data, Evas_Object *button, void *event_info)
{
   dlog_print(DLOG_DEBUG, "lesson15", "CLICK event!");
   UIData *ui = data;

   ui->click_count++;
   _label_click_count_text_set(ui);
}

static void
_button_create(UIData *ui)
{
   ui->button = elm_button_add(ui->layout);

   //  Setting a style to the widget
   elm_object_style_set(ui->button, "bottom");

   //  Enable auto-repeat events for a button
   elm_button_autorepeat_set(ui->button, EINA_TRUE);

   //Delay of the first repeat event in 3 seconds
   elm_button_autorepeat_initial_timeout_set(ui->button, 2);

   //Set the interval between repetition events in half of a second
   elm_button_autorepeat_gap_timeout_set(ui->button, 0.5);

   //  Setting a text to the widget
   elm_object_text_set(ui->button, "Click on me!");

   evas_object_smart_callback_add(ui->button, "pressed", press_cb, NULL);
   evas_object_smart_callback_add(ui->button, "unpressed", unpress_cb, NULL);
   evas_object_smart_callback_add(ui->button, "repeated", repeat_cb, NULL);
   evas_object_smart_callback_add(ui->button, "clicked", click_cb, ui);

   evas_object_show(ui->button);
}

static void
_label_create(UIData *ui)
{
   ui->label = elm_label_add(ui->layout);

   _label_click_count_text_set(ui);

   Evas_Coord w = 0;
   Evas_Coord h = 0;
   evas_object_size_hint_max_get(ui->label, &w, &h);
   evas_object_size_hint_max_set(ui->label, w, 80);

   evas_object_show(ui->label);
}

static void
_layout_create(UIData *ui)
{
   ui->layout = elm_layout_add(ui->conform);

   //Set standard  layout style: elm/layout/bottom_button/default
   elm_layout_theme_set(ui->layout, "layout", "bottom_button", "default");

   _button_create(ui);
   elm_object_part_content_set(ui->layout, "elm.swallow.button", ui->button);

   _label_create(ui);
   elm_object_part_content_set(ui->layout, "elm.swallow.content", ui->label);

   evas_object_show(ui->layout);
}

static void
_conformant_create(UIData *ui)
{
   ui->conform = elm_conformant_add(ui->win);


   //Specify to conformant occupies all allocated to him space
     // (horizontally and vertically)
   evas_object_size_hint_weight_set(ui->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

   // Create a layout and place it into a conformant
   _layout_create(ui);
   elm_object_content_set(ui->conform, ui->layout);

   evas_object_show(ui->conform);
}

static void
_window_create(UIData *ui)
{
	// Create and configure the main window
   ui->win = elm_win_util_standard_add(NULL, NULL);

   // Create a conformant - in this case, the main container in the application,
     // which is also the interface between the application and
    // system elements, such as a keyboard.
   _conformant_create(ui);

   //  Set the size of the conformant widget the same as the size of the window
   elm_win_resize_object_add(ui->win, ui->conform);

   //Register the function that will be called
     //when you press the hardware Back button
   eext_object_event_callback_add(ui->win, EEXT_CALLBACK_BACK, _win_back_cb, ui);


   // Always display the window only after the entire base
   // user interface will be displayed.
   evas_object_show(ui->win);
}

static bool
_app_create_cb(void *data)
{
   UIData *ui = data;

   _window_create(ui);

   return true;
}

static void
_app_terminate_cb(void *data)
{
   UIData *ui = data;

   //Before closing the application, delete the main widget (window),
    // then all  "children" widgets of this window (all widgets of this application) will be deleted.
   evas_object_del(ui->win);
}

int
main(int argc, char *argv[])
{
   UIData ui = {0,};
   ui_app_lifecycle_callback_s lifecycle_callbacks = {0,};

   //Register created functions to transfer them to the main loop of
   // this application.
   lifecycle_callbacks.create    = _app_create_cb;
   lifecycle_callbacks.terminate = _app_terminate_cb;

   // Calling the next function, launch the main application cycle
   return ui_app_main(argc, argv, &lifecycle_callbacks, &ui);
}
